@@ -10,7 +10,7 @@ class WeConnectError(Base):
     __tablename__ = 'weconnect_errors'
     id = Column(Integer, primary_key=True)
     datetime = Column(DatetimeDecorator(timezone=True), nullable=False)
-    errortype = Column(Enum(ErrorEventType, length=63))
+    errortype = Column(Enum(ErrorEventType))
     detail = Column(String)
 
     def __init__(self, datetime, errortype, detail):
