@@ -4,6 +4,7 @@ from io import BytesIO
 from pyqrcode import pyqrcode
 
 from flask import Blueprint, render_template, current_app, abort, redirect, url_for, request, send_file, flash
+from flask_login import login_required
 from flask_wtf import FlaskForm
 from wtforms import FieldList, FormField, StringField, IntegerField, SubmitField, HiddenField
 from wtforms.validators import DataRequired, NumberRange, Optional, Length, Regexp, ValidationError
@@ -20,25 +21,32 @@ bp = Blueprint('settings', __name__, url_prefix='/settings')
 
 class VehicleSettingsForm(FlaskForm):
     primary_capacity = IntegerField('Primary Capacity', validators=[DataRequired(), NumberRange(min=1, max=500)])
+    primary_capacity_total = IntegerField('Primary Capacity Total', validators=[Optional(), NumberRange(min=1, max=500)], filters=[lambda x: x or None])
     primary_wltp_range = IntegerField('Primary WLTP range in km', validators=[DataRequired(), NumberRange(min=1, max=1000)])
     secondary_capacity = IntegerField('Secondary Capacity', validators=[Optional(), NumberRange(min=1, max=500)], filters=[lambda x: x or None])
+    secondary_capacity_total = IntegerField('Secondary Capacity Total', validators=[Optional(), NumberRange(min=1, max=500)], filters=[lambda x: x or None])
     secondary_wltp_range = IntegerField('Secondary WLTP Range', validators=[Optional(), NumberRange(min=1, max=1000)], filters=[lambda x: x or None])
     save = SubmitField('Save')
 
 
 class ElectricVehicleSettingsForm(VehicleSettingsForm):
     primary_capacity = IntegerField('Usable Battery Capacity (net) in kWh', validators=[DataRequired(), NumberRange(min=1, max=500)])
+    primary_capacity_total = IntegerField('Total Battery Capacity (gross) in kWh', validators=[DataRequired(), NumberRange(min=1, max=500)])
     primary_wltp_range = IntegerField('WLTP range in km', validators=[DataRequired(), NumberRange(min=1, max=1000)])
     secondary_capacity = HiddenField('Secondary Capacity', validators=[Optional(), NumberRange(min=1, max=500)], filters=[lambda x: x or None])
-    secondary_wltp_range = HiddenField('Secondary WLTP Range', validators=[Optional(), NumberRange(min=1, max=1000)], filters=[lambda x: x or None])
+    secondary_capacity_total = HiddenField('Secondary Capacity Total', validators=[Optional()], filters=[lambda x: x or None])
+    secondary_wltp_range = HiddenField('Secondary WLTP Range', validators=[Optional()], filters=[lambda x: x or None])
     save = SubmitField('Save')
 
 
 class HybridVehicleSettingsForm(VehicleSettingsForm):
     primary_capacity = IntegerField('Gasoline tank size in liters', validators=[DataRequired(), NumberRange(min=1, max=500)])
+    primary_capacity_total = HiddenField('Primary Capacity Total', validators=[Optional()], filters=[lambda x: x or None])
     primary_wltp_range = IntegerField('WLTP range in km gasoline only', validators=[DataRequired(), NumberRange(min=1, max=1000)])
-    secondary_capacity = IntegerField('Usable Battery Capacity (net) in kWh', validators=[Optional(), NumberRange(min=1, max=500)],
+    secondary_capacity = IntegerField('Usable Battery Capacity (net) in kWh', validators=[DataRequired(), NumberRange(min=1, max=500)],
                                       filters=[lambda x: x or None])
+    secondary_capacity_total = IntegerField('Total Battery Capacity (gross) in kWh', validators=[DataRequired(), NumberRange(min=1, max=500)],
+                                            filters=[lambda x: x or None])
     secondary_wltp_range = IntegerField('WLTP range in km electic only', validators=[Optional(), NumberRange(min=1, max=1000)], filters=[lambda x: x or None])
     save = SubmitField('Save')
 
@@ -71,6 +79,7 @@ ABRPSettingsEntry = namedtuple('ABRPSettings', ('accounts'))
 
 
 @bp.route('/vehicle/<vin>', methods=['GET', 'POST'])
+@login_required
 def vehicle(vin):
     if vin not in current_app.weConnect.vehicles:
         abort(404, f"Vehicle with VIN {vin} doesn't exist.")
@@ -80,6 +89,7 @@ def vehicle(vin):
 
 
 @bp.route('/vehicle/database/<vin>', methods=['GET', 'POST'])
+@login_required
 def vehicleDBParameters(vin):
     if vin not in current_app.weConnect.vehicles:
         abort(404, f"Vehicle with VIN {vin} doesn't exist.")
@@ -100,13 +110,17 @@ def vehicleDBParameters(vin):
 
     if request.method == "GET":
         form.primary_capacity.data = dbVehicle.settings.primary_capacity
+        form.primary_capacity_total.data = dbVehicle.settings.primary_capacity_total
         form.primary_wltp_range.data = dbVehicle.settings.primary_wltp_range
         form.secondary_capacity.data = dbVehicle.settings.secondary_capacity
+        form.secondary_capacity_total.data = dbVehicle.settings.secondary_capacity_total
         form.secondary_wltp_range.data = dbVehicle.settings.secondary_wltp_range
     elif form.validate_on_submit():
         dbVehicle.settings.primary_capacity = form.primary_capacity.data
+        dbVehicle.settings.primary_capacity_total = form.primary_capacity_total.data
         dbVehicle.settings.primary_wltp_range = form.primary_wltp_range.data
         dbVehicle.settings.secondary_capacity = form.secondary_capacity.data
+        dbVehicle.settings.secondary_capacity_total = form.secondary_capacity_total.data
         dbVehicle.settings.secondary_wltp_range = form.secondary_wltp_range.data
         current_app.db.session.commit()
 
@@ -116,6 +130,7 @@ def vehicleDBParameters(vin):
 
 
 @bp.route('/vehicle/abrp/<vin>', methods=['GET', 'POST'])  # noqa: C901
+@login_required
 def vehicleABRPSettings(vin):  # noqa: C901
     if vin not in current_app.weConnect.vehicles:
         abort(404, f"Vehicle with VIN {vin} doesn't exist.")
@@ -159,6 +174,7 @@ def vehicleABRPSettings(vin):  # noqa: C901
 
 
 @bp.route('/homekit', methods=['GET', 'POST'])
+@login_required
 def homekit():
     if current_app.homekitDriver is None:
         abort(404, 'Homekit support was not enabled. Enabled it with --with-homekit')
@@ -183,6 +199,7 @@ def homekit():
 
 
 @bp.route('/homekit/homekit-qr.png', methods=['GET'])
+@login_required
 def homekitQR():
     if current_app.homekitDriver is None or current_app.homekitDriver.accessory is None:
         abort(404, "VWsFriend is running without Homekit support")
